package co.hyperverge.hypersnapsdk.hypersnapsdk_flutter;

import android.app.Activity;
import android.os.Build;
import android.util.Log;

import androidx.annotation.RequiresApi;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

import co.hyperverge.hypersnapsdk.listeners.APICompletionCallback;
import co.hyperverge.hypersnapsdk.network.HVNetworkHelper;
import co.hyperverge.hypersnapsdk.objects.HVError;
import co.hyperverge.hypersnapsdk.objects.HVResponse;

public class FLHVNetworkHelper {


    static final String NETWORK_HELPER_MAKE_OCR_CALL = "networkHelperMakeOCRCall";
    static final String NETWORK_HELPER_MAKE_FACE_MATCH_CALL = "networkHelperMakeFaceMatchCall";

    private static final String TAG = FLHVNetworkHelper.class.getSimpleName();

    public static JSONObject convertMapToJson(Map<String, Object> map) throws JSONException {
        JSONObject object = new JSONObject();
        ObjectMapper objectMapper = new ObjectMapper();

        try {
            String json = objectMapper.writeValueAsString(map);
            object = new JSONObject(json);
        } catch (JsonProcessingException e) {
            Log.e(TAG, e.getMessage());
        }
        return object;
    }

    public static Map<String, Object> convertJsonToMap(JSONObject jsonObject) {
        ObjectMapper mapper = new ObjectMapper();
        String jsonString = jsonObject.toString();
        Map<String, Object> map = new HashMap<>();
        try {
            map = mapper.readValue(jsonString, new TypeReference<Map<String, Object>>() {
            });

        } catch (IOException e) {
            Log.e(TAG, e.getMessage());
        }
        return map;
    }

    @RequiresApi(api = Build.VERSION_CODES.KITKAT)
    public void makeOCRCall(Activity activity, String endpoint, String documentUri, Map<String, Object> parameters, Map<String, Object> headers, final APICompletionCallback resultCallback) {

        APICompletionCallback completionCallback = new APICompletionCallback() {
            @Override
            public void onResult(HVError error, HVResponse hvResponse) {
                resultCallback.onResult(error, hvResponse);
            }
        };
        try {
            JSONObject params = new JSONObject();
            if (parameters != null) {
                params = convertMapToJson(parameters);
            }

            JSONObject headers2 = new JSONObject();
            if (headers != null) {
                headers2 = convertMapToJson(headers);
            }
            HVNetworkHelper.makeOCRCall(activity, endpoint, documentUri, params, headers2, completionCallback);
        } catch (Exception e) {
            Log.e(TAG, Objects.requireNonNull(e.getMessage()));
        }
    }

    @RequiresApi(api = Build.VERSION_CODES.KITKAT)
    public void makeFaceMatchCall(Activity activity, String endpoint, String faceUri, String documentUri, Map<String, Object> parameters, Map<String, Object> headers, final APICompletionCallback resultCallback) {
        APICompletionCallback completionCallback = new APICompletionCallback() {
            @Override
            public void onResult(HVError error, HVResponse hvResponse) {
                resultCallback.onResult(error, hvResponse);
            }
        };
        try {
            JSONObject params = new JSONObject();
            if (parameters != null) {
                params = convertMapToJson(parameters);
            }

            JSONObject headers2 = new JSONObject();
            if (headers != null) {
                headers2 = convertMapToJson(headers);
            }

            HVNetworkHelper.makeFaceMatchCall(activity, endpoint, faceUri, documentUri, params, headers2, completionCallback);
        } catch (JSONException e) {
            Log.e(TAG, Objects.requireNonNull(e.getMessage()));
        }
    }

    @RequiresApi(api = Build.VERSION_CODES.KITKAT)
    public List<Map<String, Object>> getArrayListFromOnResultNetworkHelper(HVError error, HVResponse hvResponse) {

        ArrayList<Map<String, Object>> arrayList = new ArrayList<>();


        Map<String, Object> errorObj = new HashMap<>();
        Map<String, Object> resultsObj = new HashMap<>();
        try {

            JSONObject result = hvResponse.getApiResult();
            JSONObject headers = hvResponse.getApiHeaders();

            String imageURI = hvResponse.getImageURI();
            String fullImageUri = hvResponse.getFullImageURI();
            String action = hvResponse.getAction();
            String retakeMessage = hvResponse.getRetakeMessage();

            if (error != null) {
                errorObj.put("errorCode", error.getErrorCode());
                errorObj.put("errorMessage", error.getErrorMessage());
            } else {

                if (result != null) {
                    try {
                        resultsObj = convertJsonToMap(result);
                        resultsObj.put("apiResult", result.toString());
                        resultsObj.put("imageUri", imageURI);
                        if (fullImageUri != null && !fullImageUri.isEmpty()) {
                            resultsObj.put("fullImageUri", fullImageUri);
                        }
                        if (retakeMessage != null && !retakeMessage.isEmpty()) {
                            resultsObj.put("retakeMessage", retakeMessage);
                        }
                        if (action != null && !action.isEmpty()) {
                            resultsObj.put("action", action);
                        }
                    } catch (Exception e) {
                        Log.e(TAG, Objects.requireNonNull(e.getMessage()));
                    }
                }
                if (headers != null) {
                    try {
                        resultsObj.put("apiHeaders", headers.toString());
                    } catch (Exception e) {
                        Log.e(TAG, Objects.requireNonNull(e.getMessage()));
                    }
                }

            }
        } catch (Exception e) {
            Log.e(TAG, e.getMessage());
        }
        arrayList.add(errorObj);
        arrayList.add(resultsObj);
        return arrayList;
    }
}
