package co.hyperverge.hypersnapsdk.hypersnapsdk_flutter;

import android.content.Context;
import android.util.Log;

import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;

import co.hyperverge.hypersnapsdk.HyperSnapSDK;
import co.hyperverge.hypersnapsdk.listeners.SessionStatusCallback;
import co.hyperverge.hypersnapsdk.objects.HVDocConfig;
import co.hyperverge.hypersnapsdk.objects.HVError;
import co.hyperverge.hypersnapsdk.objects.HVFaceConfig;
import co.hyperverge.hypersnapsdk.objects.HVSessionResponse;
import co.hyperverge.hypersnapsdk.objects.HyperSnapParams;

public class FLHyperSnapSDK {

    static final String INITIALIZE = "initialize";
    static final String INITIALIZE_WITH_TOKEN = "initializeWithToken";
    static final String SET_SHOULD_ENABLE_SSL_PINNING =
            "setShouldEnableSSLPinning";
    static final String SET_SHOULD_USE_SIGNATURE = "setShouldUseSignature";
    static final String SET_HTTPS_TIMEOUT_VALUES = "setHttpTimeoutValues";
    static final String SORT_JSON_KEYS_ALPHABETICALLY =
            "sortJSONKeysAlphabetically";
    static final String SET_SHOULD_RETURN_RAW_RESPONSE =
            "setShouldReturnRawResponse";
    static final String START_USER_SESSION = "startUserSession";
    static final String END_USER_SESSION = "endUserSession";
    private static final String TAG = FLHyperSnapSDK.class.getSimpleName();

    public static <K, V> SortedMap<K, V> convertToTreeMap(Map<K, V> hashMap) {
        // Create a new TreeMap
        TreeMap<K, V> treeMap = new TreeMap<>();
        // Pass the hashMap to putAll() method
        treeMap.putAll(hashMap);
        // Return the TreeMap
        return treeMap;
    }

    public void initialize(Context context, String appId, String appName, String regionValue) {
        String[] regionParams = regionValue.split("\\.");
        HyperSnapParams.Region region = getHVHyperSnapParam(regionParams[0],
                regionParams[1], HyperSnapParams.Region.class);
        HyperSnapSDK.init(context, appId, appName, region);
    }

    public void initializeWithToken(Context context, String appId, String accessToken, String regionValue) {
        HyperSnapParams.Region region = getRegion(regionValue);
        HyperSnapSDK.initWithToken(context, appId, accessToken, region);
    }

    //    @Deprecated
    public void setShouldEnableSSLPinning(boolean shouldEnable) {
        HyperSnapSDK.setShouldEnableSSLPinning(shouldEnable);
    }

    //    @Deprecated
    public void setShouldUseSignature(boolean useSignature) {
        HyperSnapSDK.setShouldUseSignature(useSignature);
    }

    //    @Deprecated
    public void setHttpTimeoutValues(int connectTimeout, int readTimeout, int writeTimeout) {
        HyperSnapSDK.setHttpTimeoutValues(connectTimeout, readTimeout, writeTimeout);
    }

    //    @Deprecated
    public String sortJSONKeysAlphabetically(Map<String, Object> map) {
        return HyperSnapSDK.sortJSONKeysAlphabetically((TreeMap<String, Object>) convertToTreeMap(map));
    }

    //    @Deprecated
    public void setShouldReturnRawResponse(boolean shouldSend) {
        HyperSnapSDK.setShouldReturnRawResponse(shouldSend);
    }

    public boolean startUserSession(String userID) {
        if (userID != null) {
            boolean state;
            HVSessionResponse hvSessionResponse;
            if (userID.isEmpty()) {
                hvSessionResponse = HyperSnapSDK.startUserSession();
                /*state = HyperSnapSDK.startUserSession(new SessionStatusCallback() {
                    @Override
                    public void onFailure(HVError hvError) {
                        Log.e(TAG, "hvError " + hvError.getErrorCode());
                    }
                });*/
            } else {
                hvSessionResponse = HyperSnapSDK.startUserSession(userID);
                /*state = HyperSnapSDK.startUserSession(userID, new SessionStatusCallback() {
                    @Override
                    public void onFailure(HVError hvError) {
                        Log.e(TAG, "hvError " + hvError.getErrorCode());
                    }
                });*/
            }
            state = hvSessionResponse.isSuccess();
            return state;
        } else {
            return false;
        }
    }

    public void endUserSession() {
        HyperSnapSDK.endUserSession();
    }

    public <T> T getHVHyperSnapParam(String enumClass, String param, Class<T> type) {
        switch (enumClass) {
            case "Product": {
                return type.cast(HyperSnapParams.Product.valueOf(param));
            }
            case "Region": {
                return type.cast(HyperSnapParams.Region.valueOf(param));
            }
            case "Document": {
                return type.cast(HVDocConfig.Document.valueOf(param));
            }
            case "LivenessMode": {
                return type.cast(HVFaceConfig.LivenessMode.valueOf(param));
            }
            default: {
                return null;
            }
        }
    }

    public HyperSnapParams.Region getRegion(String regionValue) {
        String[] regionParams = regionValue.split("\\.");
        return getHVHyperSnapParam(regionParams[0],
                regionParams[1], HyperSnapParams.Region.class);
    }
}
