package co.hyperverge.hypersnapsdk.hypersnapsdk_flutter;


import android.app.Activity;
import android.content.Context;
import android.os.Build;
import android.util.Log;

import androidx.annotation.NonNull;
import androidx.annotation.RequiresApi;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import co.hyperverge.hypersnapsdk.listeners.APICompletionCallback;
import co.hyperverge.hypersnapsdk.listeners.DocCaptureCompletionHandler;
import co.hyperverge.hypersnapsdk.listeners.FaceCaptureCompletionHandler;
import co.hyperverge.hypersnapsdk.listeners.QRScannerCompletionHandler;
import co.hyperverge.hypersnapsdk.objects.HVError;
import co.hyperverge.hypersnapsdk.objects.HVResponse;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

/**
 * HypersnapsdkFlutterPlugin
 */
public class HypersnapsdkFlutterPlugin implements FlutterPlugin, ActivityAware {

    private static final String TAG = HypersnapsdkFlutterPlugin.class.getSimpleName();
    private static final String ERROR_OBJ = "errorObj";
    private static final String RESULT_OBJ = "resultObj";
    private static final String HEADERS = "headers";

    private final FLHyperSnapSDK flHypersnapSDK = new FLHyperSnapSDK();
    private final FLHyperSnapParams flHyperSnapParams = new FLHyperSnapParams();
    private final FLHVDocsCapture flhvDocsCapture = new FLHVDocsCapture();
    private final FLHVFaceCapture flhvFaceCapture = new FLHVFaceCapture();
    private final FLHVQRScanCapture flhvqrScanCapture = new FLHVQRScanCapture();
    private final FLHVNetworkHelper flhvNetworkHelper = new FLHVNetworkHelper();

    private final MethodCallHandler hyperSnapParamsMethodCallHandler = new MethodCallHandler() {
        @Override
        public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
            if (call.method.equalsIgnoreCase(FLHyperSnapParams.GET_VALID_PARAMS)) {
                result.success(flHyperSnapParams.getConstants());
            } else {
                result.notImplemented();
            }
        }
    };
    protected Context context;
    protected Activity activity;
    private final MethodCallHandler hyperSnapSDKMethodCallHandler = new MethodCallHandler() {
        @Override
        public void onMethodCall(@NonNull MethodCall call, @NonNull final Result result) {
            if (call.method.equalsIgnoreCase(FLHyperSnapSDK.INITIALIZE)) {
                String appID = call.argument("appID");
                String appKey = call.argument("appKey");
                String region = call.argument("region");
                flHypersnapSDK.initialize(activity, appID, appKey, region);
                result.success("Initialised");
            } else if (call.method.equalsIgnoreCase(FLHyperSnapSDK.INITIALIZE_WITH_TOKEN)) {
                String appID = call.argument("appID");
                String accessToken = call.argument("accessToken");
                String region = call.argument("region");
                flHypersnapSDK.initializeWithToken(activity, appID, accessToken, region);
                result.success("Initialised");
            } else if (call.method.equalsIgnoreCase(FLHyperSnapSDK.SET_SHOULD_ENABLE_SSL_PINNING)) {
                boolean shouldEnableSSLPinning = call.argument("shouldEnableSSLPinning");
                flHypersnapSDK.setShouldEnableSSLPinning(shouldEnableSSLPinning);
            } else if (call.method.equalsIgnoreCase(FLHyperSnapSDK.SET_SHOULD_USE_SIGNATURE)) {
                boolean shouldUseSignature = call.argument("shouldUseSignature");
                flHypersnapSDK.setShouldUseSignature(shouldUseSignature);
            } else if (call.method.equalsIgnoreCase(FLHyperSnapSDK.SET_HTTPS_TIMEOUT_VALUES)) {
                int connectTimeout = call.argument("connectTimeout");
                int readTimeout = call.argument("readTimeout");
                int writeTimeout = call.argument("writeTimeout");
                flHypersnapSDK.setHttpTimeoutValues(connectTimeout, readTimeout, writeTimeout);
            } else if (call.method.equalsIgnoreCase(FLHyperSnapSDK.SORT_JSON_KEYS_ALPHABETICALLY)) {
                Map<String, Object> map = call.argument("map");
                result.success(flHypersnapSDK.sortJSONKeysAlphabetically(map));
            } else if (call.method.equalsIgnoreCase(FLHyperSnapSDK.SET_SHOULD_RETURN_RAW_RESPONSE)) {
                boolean shouldReturnRawResponse = call.argument("shouldReturnRawResponse");
                flHypersnapSDK.setShouldReturnRawResponse(shouldReturnRawResponse);
            } else if (call.method.equalsIgnoreCase(FLHyperSnapSDK.START_USER_SESSION)) {
                String userID = call.argument("userID");
                result.success(flHypersnapSDK.startUserSession(userID));
            } else if (call.method.equalsIgnoreCase(FLHyperSnapSDK.END_USER_SESSION)) {
                flHypersnapSDK.endUserSession();
            } else {
                result.notImplemented();
            }

        }
    };
    private final MethodCallHandler hvDocsCaptureMethodCallHandler = new MethodCallHandler() {
        @Override
        public void onMethodCall(@NonNull MethodCall call, @NonNull final Result result) {
            if (call.method.equalsIgnoreCase(FLHVDocsCapture.DOCS_CAPTURE_SET_OCR_API_DETAILS)) {
                String ocrEndpoint = call.argument("ocrEndpoint");
                String documentSideString = call.argument("documentSideString");
                Map<String, Object> params = call.argument("params");
                Map<String, Object> headers = call.argument(HEADERS);

                try {
                    flhvDocsCapture.setOCRAPIDetails(ocrEndpoint, documentSideString, params, headers);
                } catch (JSONException e) {
                    Log.e(TAG, e.getMessage());
                }
            } else if (call.method.equalsIgnoreCase(FLHVDocsCapture.DOCS_CAPTURE_SET_ASPECT_RATIO)) {
                double aspectRatioValue = call.argument("aspectRatioValue");
                flhvDocsCapture.setAspectRatio((float) aspectRatioValue);
            } else if (call.method.equalsIgnoreCase(FLHVDocsCapture.DOCS_CAPTURE_SET_SHOULD_ADD_PADDING)) {
                boolean shouldSetPadding = call.argument("shouldSetPadding");
                flhvDocsCapture.setShouldAddPadding(shouldSetPadding);
            } else if (call.method.equalsIgnoreCase(FLHVDocsCapture.DOCS_CAPTURE_SET_SHOULD_SHOW_FLASH_ICON)) {
                boolean shouldShowFlashIcon = call.argument("shouldShowFlashIcon");
                flhvDocsCapture.setShouldShowFlashIcon(shouldShowFlashIcon);
            } else if (call.method.equalsIgnoreCase(FLHVDocsCapture.DOCS_CAPTURE_SET_SHOULD_SHOW_REVIEW_SCREEN)) {
                boolean shouldShowReviewScreen = call.argument("shouldShowReviewScreen");
                flhvDocsCapture.setShouldShowReviewScreen(shouldShowReviewScreen);
            } else if (call.method.equalsIgnoreCase(FLHVDocsCapture.DOCS_CAPTURE_SET_DOC_CAPTURE_SUB_TEXT)) {
                String subText = call.argument("subText");
                flhvDocsCapture.setDocCaptureSubText(subText);
            } else if (call.method.equalsIgnoreCase(FLHVDocsCapture.DOCS_CAPTURE_SET_DOC_CAPTURE_DESCRIPTION)) {
                String description = call.argument("description");
                flhvDocsCapture.setDocCaptureDescription(description);
            } else if (call.method.equalsIgnoreCase(FLHVDocsCapture.DOCS_CAPTURE_SET_SHOULD_SHOW_INSTRUCTION_PAGE)) {
                boolean shouldShowInstructionPage = call.argument("shouldShowInstructionPage");
                flhvDocsCapture.setShouldShowInstructionPage(shouldShowInstructionPage);
            } else if (call.method.equalsIgnoreCase(FLHVDocsCapture.DOCS_CAPTURE_SET_DOCUMENT_TYPE)) {
                String docTypeValue = call.argument("docTypeValue");
                flhvDocsCapture.setDocumentType(docTypeValue);
            } else if (call.method.equalsIgnoreCase(FLHVDocsCapture.DOCS_CAPTURE_SET_DOC_CAPTURE_TITLE)) {
                String titleText = call.argument("titleText");
                flhvDocsCapture.setDocCaptureTitle(titleText);
            } else if (call.method.equalsIgnoreCase(FLHVDocsCapture.DOCS_CAPTURE_SET_DOC_REVIEW_TITLE)) {
                String docReviewTitle = call.argument("docReviewTitle");
                flhvDocsCapture.setDocReviewTitle(docReviewTitle);
            } else if (call.method.equalsIgnoreCase(FLHVDocsCapture.DOCS_CAPTURE_SET_DOC_REVIEW_DESCRIPTION)) {
                String docReviewDescription = call.argument("docReviewDescription");
                flhvDocsCapture.setDocReviewDescription(docReviewDescription);
            } else if (call.method.equalsIgnoreCase(FLHVDocsCapture.DOCS_CAPTURE_SET_SHOULD_ENABLE_RETRIES)) {
                boolean shouldEnableRetries = call.argument("shouldEnableRetries");
                flhvDocsCapture.setShouldEnableRetries(shouldEnableRetries);
            } else if (call.method.equalsIgnoreCase(FLHVDocsCapture.DOCS_CAPTURE_START)) {
                DocCaptureCompletionHandler callback = new DocCaptureCompletionHandler() {
                    @Override
                    public void onResult(HVError hvError, HVResponse hvResponse) {
                        ArrayList<Map<String, Object>> arr = (ArrayList<Map<String, Object>>) flhvDocsCapture.getMapsFromOnResult(hvError, hvResponse);
                        Map<String, Object> map = new HashMap<>();
                        map.put(ERROR_OBJ, arr.get(0));
                        map.put(RESULT_OBJ, arr.get(1));
                        result.success(map);
                    }
                };
                flhvDocsCapture.start(activity, callback);
            } else if (call.method.equalsIgnoreCase(FLHVDocsCapture.DOCS_CAPTURE_SET_UI_STRINGS)) {
                Map<String, Object> uiStrings = call.argument("uiStrings");
                flhvDocsCapture.setUIStrings(uiStrings);
            } else {
                result.notImplemented();
            }
        }
    };
    private final MethodCallHandler hvFaceCaptureMethodCallHandler = new MethodCallHandler() {
        @RequiresApi(api = Build.VERSION_CODES.KITKAT)
        @Override
        public void onMethodCall(@NonNull MethodCall call, @NonNull final Result result) {
            if (call.method.equalsIgnoreCase(FLHVFaceCapture.FACE_CAPTURE_START)) {
                FaceCaptureCompletionHandler callback = new FaceCaptureCompletionHandler() {
                    @Override
                    public void onResult(HVError hvError, HVResponse hvResponse) {
                        ArrayList<Map<String, Object>> arr = (ArrayList<Map<String, Object>>) flhvDocsCapture.getMapsFromOnResult(hvError, hvResponse);
                        Map<String, Object> map = new HashMap<>();
                        map.put(ERROR_OBJ, arr.get(0));
                        map.put(RESULT_OBJ, arr.get(1));
                        result.success(map);
                    }
                };
                flhvFaceCapture.start(activity, callback);
            } else if (call.method.equalsIgnoreCase(FLHVFaceCapture.FACE_CAPTURE_SET_LIVENESS_ENDPOINT)) {
                String livenessEndpoint = call.argument("livenessEndpoint");
                flhvFaceCapture.setLivenessEndpoint(livenessEndpoint);
            } else if (call.method.equalsIgnoreCase(FLHVFaceCapture.FACE_CAPTURE_SET_SHOULD_RETURN_FULL_IMAGE_URL)) {
                boolean shouldReturnFullImageUrl = call.argument("shouldReturnFullImageUrl");
                flhvFaceCapture.setShouldReturnFullImageUrl(shouldReturnFullImageUrl);
            } else if (call.method.equalsIgnoreCase(FLHVFaceCapture.FACE_CAPTURE_SET_SHOULD_SHOW_CAMERA_SWITCH_BUTTON)) {
                boolean shouldShow = call.argument("shouldShow");
                flhvFaceCapture.setShouldShowCameraSwitchButton(shouldShow);
            } else if (call.method.equalsIgnoreCase(FLHVFaceCapture.FACE_CAPTURE_SET_SHOULD_SHOW_INSTRUCTION_PAGE)) {
                boolean shouldShowInstructionPage = call.argument("shouldShowInstructionPage");
                flhvFaceCapture.setShouldShowInstructionPage(shouldShowInstructionPage);
            } else if (call.method.equalsIgnoreCase(FLHVFaceCapture.FACE_CAPTURE_SET_LIVENESS_MODE)) {
                String livenessValue = call.argument("livenessValue");
                flhvFaceCapture.setLivenessMode(livenessValue);
            } else if (call.method.equalsIgnoreCase(FLHVFaceCapture.FACE_CAPTURE_SET_FACE_CAPTURE_TITLE)) {
                String faceCaptureTitle = call.argument("faceCaptureTitle");
                flhvFaceCapture.setFaceCaptureTitle(faceCaptureTitle);
            } else if (call.method.equalsIgnoreCase(FLHVFaceCapture.FACE_CAPTURE_SET_SHOULD_USE_BACK_CAMERA)) {
                boolean shouldUseBackCamera = call.argument("shouldUseBackCamera");
                flhvFaceCapture.setShouldUseBackCamera(shouldUseBackCamera);
            } else if (call.method.equalsIgnoreCase(FLHVFaceCapture.FACE_CAPTURE_SET_SHOULD_ADD_PADDING)) {
                boolean shouldSetPadding = call.argument("shouldSetPadding");
                flhvFaceCapture.setShouldAddPadding(shouldSetPadding);
            } else if (call.method.equalsIgnoreCase(FLHVFaceCapture.FACE_CAPTURE_SET_PADDING)) {
                double leftPadding = call.argument("leftPadding");
                double rightPadding = call.argument("rightPadding");
                double topPadding = call.argument("topPadding");
                double bottomPadding = call.argument("bottomPadding");
                flhvFaceCapture.setPadding(leftPadding, rightPadding, topPadding, bottomPadding);
            } else if (call.method.equalsIgnoreCase(FLHVFaceCapture.FACE_CAPTURE_SET_LIVENESS_API_PARAMETERS)) {
                String params = call.argument("params");
                flhvFaceCapture.setLivenessAPIParameters(params);
            } else if (call.method.equalsIgnoreCase(FLHVFaceCapture.FACE_CAPTURE_SET_LIVENESS_API_HEADERS)) {
                String headers = call.argument(HEADERS);
                flhvFaceCapture.setLivenessAPIHeaders(headers);
            } else if (call.method.equalsIgnoreCase(FLHVFaceCapture.FACE_CAPTURE_SET_SHOULD_HANDLE_RETRIES)) {
                boolean shouldHandleRetries = call.argument("shouldHandleRetries");
                flhvFaceCapture.setShouldHandleRetries(shouldHandleRetries);
            } else if (call.method.equalsIgnoreCase(FLHVFaceCapture.FACE_CAPTURE_SET_UI_STRINGS)) {
                Map<String, Object> uiStrings = call.argument("uiStrings");
                flhvFaceCapture.setUIStrings(uiStrings);
            } else {
                result.notImplemented();
            }
        }
    };
    private final MethodCallHandler hvNetworkHelperMethodCallHandler = new MethodCallHandler() {
        @RequiresApi(api = Build.VERSION_CODES.KITKAT)
        @Override
        public void onMethodCall(@NonNull MethodCall call, @NonNull final Result result) {
            if (call.method.equalsIgnoreCase(FLHVNetworkHelper.NETWORK_HELPER_MAKE_OCR_CALL)) {
                APICompletionCallback callback = new APICompletionCallback() {
                    @Override
                    public void onResult(HVError hvError, HVResponse hvResponse) {
                        ArrayList<Map<String, Object>> arr = (ArrayList<Map<String, Object>>) flhvNetworkHelper.getArrayListFromOnResultNetworkHelper(hvError, hvResponse);

                        Map<String, Object> map = new HashMap<>();
                        map.put(ERROR_OBJ, arr.get(0));
                        map.put(RESULT_OBJ, arr.get(1));
                        result.success(map);
                    }
                };
                String endpoint = call.argument("endpoint");
                String documentUri = call.argument("documentUri");
                Map<String, Object> parameters = call.argument("parameters");
                Map<String, Object> headers = call.argument(HEADERS);
                flhvNetworkHelper.makeOCRCall(activity, endpoint, documentUri, parameters, headers, callback);
            } else if (call.method.equalsIgnoreCase(FLHVNetworkHelper.NETWORK_HELPER_MAKE_FACE_MATCH_CALL)) {
                APICompletionCallback callback = new APICompletionCallback() {
                    @Override
                    public void onResult(HVError hvError, HVResponse hvResponse) {
                        ArrayList<Map<String, Object>> arr = (ArrayList<Map<String, Object>>) flhvNetworkHelper.getArrayListFromOnResultNetworkHelper(hvError, hvResponse);
                        Map<String, Object> map = new HashMap<>();
                        map.put(ERROR_OBJ, arr.get(0));
                        map.put(RESULT_OBJ, arr.get(1));
                        result.success(map);
                    }
                };
                String endpoint = call.argument("endpoint");
                String documentUri = call.argument("documentUri");
                String faceUri = call.argument("faceUri");
                Map<String, Object> parameters = call.argument("parameters");
                Map<String, Object> headers = call.argument(HEADERS);
                flhvNetworkHelper.makeFaceMatchCall(activity, endpoint, documentUri, faceUri, parameters, headers, callback);
            } else {
                result.notImplemented();
            }
        }
    };
    private final MethodCallHandler hvQRScanCaptureMethodCallHandler = new MethodCallHandler() {
        @Override
        public void onMethodCall(@NonNull MethodCall call, @NonNull final Result result) {
            if (call.method.equalsIgnoreCase(FLHVQRScanCapture.QR_SCAN_CAPTURE_START)) {
                QRScannerCompletionHandler callback = new QRScannerCompletionHandler() {
                    @RequiresApi(api = Build.VERSION_CODES.KITKAT)
                    @Override
                    public void onResult(HVError hvError, JSONObject hvResponse) {
                        ArrayList<Map<String, Object>> arr = (ArrayList<Map<String, Object>>) flhvqrScanCapture.getALFromOnResultQRScan(hvError, hvResponse);
                        Map<String, Object> map = new HashMap<>();
                        map.put(ERROR_OBJ, arr.get(0));
                        map.put(RESULT_OBJ, arr.get(1));
                        result.success(map);
                    }
                };
                flhvqrScanCapture.start(activity, callback);
            } else {
                result.notImplemented();
            }
        }
    };

    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private MethodChannel hyperSnapSDKChannel;
    private MethodChannel hyperSnapParamsChannel;
    private MethodChannel hvDocsCaptureChannel;
    private MethodChannel hvFaceCaptureChannel;
    private MethodChannel hvNetworkHelperChannel;
    private MethodChannel hvQRScanCaptureChannel;

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
        this.context = flutterPluginBinding.getApplicationContext();
        hyperSnapSDKChannel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "hyperSnapSDK");
        hyperSnapParamsChannel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "hyperSnapParams");
        hvDocsCaptureChannel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "hvDocsCapture");
        hvFaceCaptureChannel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "hvFaceCapture");
        hvNetworkHelperChannel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "hvNetworkHelper");
        hvQRScanCaptureChannel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "hvQRScanCapture");

        hyperSnapSDKChannel.setMethodCallHandler(hyperSnapSDKMethodCallHandler);
        hyperSnapParamsChannel.setMethodCallHandler(hyperSnapParamsMethodCallHandler);
        hvDocsCaptureChannel.setMethodCallHandler(hvDocsCaptureMethodCallHandler);
        hvFaceCaptureChannel.setMethodCallHandler(hvFaceCaptureMethodCallHandler);
        hvNetworkHelperChannel.setMethodCallHandler(hvNetworkHelperMethodCallHandler);
        hvQRScanCaptureChannel.setMethodCallHandler(hvQRScanCaptureMethodCallHandler);
    }


    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        hyperSnapSDKChannel.setMethodCallHandler(null);
        hyperSnapParamsChannel.setMethodCallHandler(null);
        hvDocsCaptureChannel.setMethodCallHandler(null);
        hvFaceCaptureChannel.setMethodCallHandler(null);
        hvNetworkHelperChannel.setMethodCallHandler(null);
        hvQRScanCaptureChannel.setMethodCallHandler(null);
    }

    @Override
    public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
        this.activity = binding.getActivity();
    }

    @Override
    public void onDetachedFromActivityForConfigChanges() {
        // Use this function only when necessary
    }

    @Override
    public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {
        // Use this function only when necessary
    }

    @Override
    public void onDetachedFromActivity() {
        // Use this function only when necessary
    }
}
