//
//  FLHVFaceCapture.swift
//

import Foundation
import HyperSnapSDK

public class FLHVFaceCapture{
    
    static var FACE_CAPTURE_START = "faceCaptureStart";
    static var FACE_CAPTURE_SET_LIVENESS_ENDPOINT =
        "faceCaptureSetLivenessEndpoint";
    static var FACE_CAPTURE_SET_SHOULD_RETURN_FULL_IMAGE_URL =
        "faceCaptureSetShouldReturnFullImageUrl";
    static var FACE_CAPTURE_SET_SHOULD_SHOW_CAMERA_SWITCH_BUTTON =
        "faceCaptureSetShouldShowCameraSwitchButton";
    static var FACE_CAPTURE_SET_SHOULD_SHOW_INSTRUCTION_PAGE =
        "faceCaptureSetShouldShowInstructionPage";
    static var FACE_CAPTURE_SET_LIVENESS_MODE =
        "faceCaptureSetLivenessMode";
    static var FACE_CAPTURE_SET_FACE_CAPTURE_TITLE =
        "faceCaptureSetFaceCaptureTitle";
    static var FACE_CAPTURE_SET_SHOULD_USE_BACK_CAMERA =
        "faceCaptureSetShouldUseBackCamera";
    static var FACE_CAPTURE_SET_SHOULD_ADD_PADDING =
        "faceCaptureSetShouldAddPadding";
    static var FACE_CAPTURE_SET_PADDING = "faceCaptureSetPadding";
    static var FACE_CAPTURE_SET_LIVENESS_API_PARAMETERS =
        "faceCaptureSetLivenessAPIParameters";
    static var FACE_CAPTURE_SET_LIVENESS_API_HEADERS =
        "faceCaptureSetLivenessAPIHeaders";
    static var FACE_CAPTURE_SET_SHOULD_HANDLE_RETRIES =
        "faceCaptureSetShouldHandleRetries";
    static var FACE_CAPTURE_SET_UI_STRINGS = "faceCaptureSetUIStrings";
    
    static var faceConfig = HVFaceConfig()
    static var liveness: String?
    static var livenessMode: HyperSnapParams.LivenessMode?
    
    public static func getFaceConfig() -> HVFaceConfig {
        return FLHVFaceCapture.faceConfig
    }
    
    public static func setLivenessEndpoint(livenessEndpoint: String){
        FLHVFaceCapture.getFaceConfig().setLivenessEndpoint(livenessEndpoint)
    }
    
    public static func setShouldShowCameraSwitchButton(shouldShow: Bool){
        getFaceConfig().setShouldShowCameraSwitchButton(shouldShow)
    }
    
    public static func setShouldShowInstructionPage(shouldShowInstructionPage: Bool){
        getFaceConfig().setShouldShowInstructionsPage(shouldShowInstructionPage)
    }
    
    public static func setLivenessMode(livenessValue: String){
        liveness = livenessValue.components(separatedBy: ".")[1]
        
        switch liveness {
        case "none":
            livenessMode = HyperSnapParams.LivenessMode.none
        case "textureLiveness":
            livenessMode = HyperSnapParams.LivenessMode.textureLiveness
        default:
            livenessMode = HyperSnapParams.LivenessMode.textureLiveness
        }
        
        getFaceConfig().setLivenessMode(livenessMode!)
    }
    
    public static func setFaceCaptureTitle(faceCaptureTitle: String){
        getFaceConfig().textConfig.setFaceCaptureTitle(faceCaptureTitle)
    }
    
    public static func setShouldUseBackCamera(shouldUseBackCamera: Bool){
        getFaceConfig().setShouldUseBackCamera(shouldUseBackCamera)
    }
    
    public static func setShouldAddPadding(shouldSetPadding: Bool){
        getFaceConfig().setShouldEnablePadding(shouldSetPadding)
    }
    public static func setShouldReturnFullImageUri(shouldReturnFullImageUri: Bool){
        getFaceConfig().setShouldReturnFullImageUri(shouldReturnFullImageUri)
    }
    public static func setShouldHandleRetries(shouldHandleRetries: Bool){
        getFaceConfig().setShouldHandleRetries(shouldHandleRetries)
    }
    public static func setPadding(leftPadding: Double, rightPadding: Double, topPadding: Double, bottomPadding: Double){
        getFaceConfig().setPadding(left: Float(leftPadding), right: Float(rightPadding), top: Float(topPadding), bottom: Float(bottomPadding))
        
    }
    
    public static func setLivenessAPIParameters(params: String){
        
        let jsonData = params.data(using: .utf8)!
        do{
            if let jsonObject = try JSONSerialization.jsonObject(with: jsonData, options: .allowFragments) as? [String : AnyObject]{
                getFaceConfig().setLivenessAPIParameters(jsonObject)
            }
        } catch let error as NSError {
            print(error)
        }
    }
    
    public static func setLivenessAPIHeaders(headers: String){
        let jsonData = headers.data(using: .utf8)!
        do{
            if let jsonObject = try JSONSerialization.jsonObject(with: jsonData, options: .allowFragments) as? [String : String]{
                getFaceConfig().setLivenessAPIHeaders(jsonObject)
            }
        } catch let error as NSError {
            print(error)
        }
    }
    
    public static func setUIStrings(uiStrings: Dictionary<String,Any>){
        if let keyExists = uiStrings["faceCaptureTitle"] {
            getFaceConfig().textConfig.setFaceCaptureTitle(keyExists as! String)
        }
        if let keyExists = uiStrings["faceCaptureMoveCloser"] {
            getFaceConfig().textConfig.setFaceCaptureMoveAwayText2(keyExists as! String)
        }
        if let keyExists = uiStrings["faceCaptureFaceFound"] {
            getFaceConfig().textConfig.setFaceCaptureFaceFoundText(keyExists as! String)
        }
        if let keyExists = uiStrings["faceCaptureFaceNotFound"] {
            getFaceConfig().textConfig.setFaceCaptureFaceNotFoundText(keyExists as! String)
        }
        if let keyExists = uiStrings["faceCaptureMoveAway"] {
            getFaceConfig().textConfig.setFaceCaptureMoveAwayText(keyExists as! String)
        }
        if let keyExists = uiStrings["faceCaptureMultipleFaces"] {
            getFaceConfig().textConfig.setFaceCaptureMultipleFacesDetectedText(keyExists as! String)
        }
        if let keyExists = uiStrings["faceRetakeTitleText"] {
            getFaceConfig().textConfig.setFaceRetakeTitleText(keyExists as! String)
        }
        if let keyExists = uiStrings["faceRetakeButtonText"] {
            getFaceConfig().textConfig.setFaceRetakeButtonText(keyExists as! String)
        }
        if let keyExists = uiStrings["faceInstructionsTitle"] {
            getFaceConfig().textConfig.setFaceInstructionsTitle(keyExists as! String)
        }
        if let keyExists = uiStrings["faceInstructionsTop1"] {
            getFaceConfig().textConfig.setFaceInstructionsTop1(keyExists as! String)
        }
        if let keyExists = uiStrings["faceInstructionsTop2"] {
            getFaceConfig().textConfig.setFaceInstructionsTop2(keyExists as! String)
        }
        if let keyExists = uiStrings["faceInstructionsNoGlasses"] {
            getFaceConfig().textConfig.setFaceInstructionsNoGlasses(keyExists as! String)
        }
        if let keyExists = uiStrings["faceInstructionsBrightLight"] {
            getFaceConfig().textConfig.setFaceInstructionsBrightLight(keyExists as! String)
        }
        if let keyExists = uiStrings["faceInstructionsNoHat"] {
            getFaceConfig().textConfig.setFaceInstructionsNoHat(keyExists as! String)
        }
        if let keyExists = uiStrings["faceInstructionsProceed"] {
            getFaceConfig().textConfig.setFaceInstructionsProceed(keyExists as! String)
        }
        if let keyExists = uiStrings["faceInstructionsProceedBackCam"] {
            getFaceConfig().textConfig.setFaceInstructionsProceedBackCam(keyExists as! String)
        }
        
    }
    
    
    public static func start(context: UIViewController, callback: @escaping ((HVError?,HVResponse?,UIViewController)->Void)) {
        
        HVFaceViewController.start(context, hvFaceConfig: getFaceConfig(), completionHandler: callback)
    }
}
