import Flutter
import UIKit
import HyperSnapSDK


public class SwiftHypersnapsdkFlutterPlugin: NSObject, FlutterPlugin {
    
    var flHypersnapSDK = FLHyperSnapSDK()
    var flHyperSnapParams = FLHyperSnapParams()
    var flhvDocsCapture =  FLHVDocsCapture()
    var flhvFaceCapture = FLHVFaceCapture()
    var flhvqrScanCapture = FLHVQRScanCapture()
    var flhvNetworkHelper = FLHVNetworkHelper()
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        
        let hyperSnapSDKChannel = FlutterMethodChannel(name: "hyperSnapSDK", binaryMessenger: registrar.messenger())
        let hyperSnapParamsChannel = FlutterMethodChannel(name: "hyperSnapParams", binaryMessenger: registrar.messenger())
        let hvDocsCaptureChannel = FlutterMethodChannel(name: "hvDocsCapture", binaryMessenger: registrar.messenger())
        let hvFaceCaptureChannel = FlutterMethodChannel(name: "hvFaceCapture", binaryMessenger: registrar.messenger())
        let hvNetworkHelperChannel = FlutterMethodChannel(name: "hvNetworkHelper", binaryMessenger: registrar.messenger())
        let hvQRScanCaptureChannel = FlutterMethodChannel(name: "hvQRScanCapture", binaryMessenger: registrar.messenger())
        
        
        
        // Handler for HyperSnapSDK
        hyperSnapSDKChannel.setMethodCallHandler({
            (call: FlutterMethodCall, result:@escaping FlutterResult) -> Void in
            
            if (call.method.caseInsensitiveCompare(FLHyperSnapSDK.INITIALIZE) == .orderedSame){
                guard let args = call.arguments as? Dictionary<String, Any> else {
                    //result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
                    return
                }
                let appID = args["appID"] as? String
                let appKey = args["appKey"] as? String
                let region = args["region"] as? String
                
                FLHyperSnapSDK.initialize(appId: appID!, appName: appKey!, regionValue: region!)
                result(String("Initialised"))
                
            } else if (call.method.caseInsensitiveCompare(FLHyperSnapSDK.INITIALIZE_WITH_TOKEN) == .orderedSame){
                guard let args = call.arguments as? Dictionary<String, Any> else {
                    //result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
                    return
                }
                let appID = args["appID"] as? String
                let accessToken = args["accessToken"] as? String
                let region = args["region"] as? String
                
                FLHyperSnapSDK.initializeWithToken(appId: appID!, accessToken: accessToken!, regionValue: region!)
                result(String("Initialised"))
                
            } else if (call.method.caseInsensitiveCompare(FLHyperSnapSDK.SET_SHOULD_USE_SIGNATURE) == .orderedSame){
                guard let args = call.arguments as? Dictionary<String, Any> else {
                    return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
                }
                
                let shouldUseSignature = args["shouldUseSignature"] as! Bool
                FLHyperSnapSDK.setShouldUseSignature(shouldUseSignature)
            }else if (call.method.caseInsensitiveCompare(FLHyperSnapSDK.SET_SHOULD_ENABLE_SSL_PINNING) == .orderedSame){
                guard let args = call.arguments as? Dictionary<String, Any> else {
                    return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
                }
                
                let shouldEnableSSLPinning = args["shouldEnableSSLPinning"] as! Bool
                FLHyperSnapSDK.setShouldEnableSSLPinning(shouldEnableSSLPinning)
            }
            else if (call.method.caseInsensitiveCompare(FLHyperSnapSDK.SET_HTTPS_TIMEOUT_VALUES) == .orderedSame){
                guard let args = call.arguments as? Dictionary<String, Any> else {
                    //result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
                    return
                }
                let readTimeout = args["map"] as! Int
                FLHyperSnapSDK.setTimeoutIntervalForRequest(Double(readTimeout))
            }else if (call.method.caseInsensitiveCompare(FLHyperSnapSDK.SORT_JSON_KEYS_ALPHABETICALLY) == .orderedSame){
                guard let args = call.arguments as? Dictionary<String, Any> else {
                    return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
                }
                let map = args["map"] as! Dictionary<String,AnyObject>
                result(FLHyperSnapSDK.sortDictionaryAlphabetically(map))
            }else if (call.method.caseInsensitiveCompare(FLHyperSnapSDK.SET_SHOULD_RETURN_RAW_RESPONSE) == .orderedSame){
                guard let args = call.arguments as? Dictionary<String, Any> else {
                    return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
                }
                let shouldReturnRawResponse = args["shouldReturnRawResponse"] as! Bool
                FLHyperSnapSDK.setShouldReturnRawResponse(shouldReturnRawResponse)
                
            }
            else if (call.method.caseInsensitiveCompare(FLHyperSnapSDK.START_USER_SESSION) == .orderedSame){
                guard let args = call.arguments as? Dictionary<String, Any> else {
                    //result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
                    return
                }
                let userID = args["userID"] as? String
                let hvError = FLHyperSnapSDK.startUserSession(userID: userID!)
                if hvError != nil{
                    result(false)
                } else {
                    result(true)
                }
            } else if (call.method.caseInsensitiveCompare(FLHyperSnapSDK.END_USER_SESSION) == .orderedSame){
                FLHyperSnapSDK.endUserSession()
            } else {
                result(FlutterMethodNotImplemented)
            }
        })
        
        
        
        // Handler for HyperSnapParams
        hyperSnapParamsChannel.setMethodCallHandler({
            (call: FlutterMethodCall, result: FlutterResult) -> Void in
            
            if (call.method.caseInsensitiveCompare(FLHyperSnapParams.GET_VALID_PARAMS) == .orderedSame){
                result(FLHyperSnapParams.getConstants())
            }else{
                result(FlutterMethodNotImplemented)
            }
        })
        
        
        
        
        // Handler for FLHVDocsCapture
        hvDocsCaptureChannel.setMethodCallHandler({
            (call: FlutterMethodCall, result: @escaping FlutterResult) -> Void in
            
            if (call.method.caseInsensitiveCompare(FLHVDocsCapture.DOCS_CAPTURE_SET_OCR_API_DETAILS) == .orderedSame){
                guard let args = call.arguments as? Dictionary<String, Any> else {
                    return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
                }
                
                let ocrEndpoint = args["ocrEndpoint"] as! String
                let documentSideString = args["documentSideString"] as! String
                let params = args["params"] as! Dictionary<String, Any>
                let headers = args["headers"] as! Dictionary<String, Any>
                
                FLHVDocsCapture.setOCRAPIDetails(ocrEndpoint: ocrEndpoint, documentSideString: documentSideString, params: params, headers: headers)
            }
            else if (call.method.caseInsensitiveCompare(FLHVDocsCapture.DOCS_CAPTURE_SET_ASPECT_RATIO) == .orderedSame){
                guard let args = call.arguments as? Dictionary<String, Any> else {
                    return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
                }
                
                let aspectRatioValue = args["aspectRatioValue"] as! Double
                FLHVDocsCapture.setAspectRatio(aspectRatioValue: aspectRatioValue)
            }
            else if (call.method.caseInsensitiveCompare(FLHVDocsCapture.DOCS_CAPTURE_SET_SHOULD_ADD_PADDING) == .orderedSame){
                guard let args = call.arguments as? Dictionary<String, Any> else {
                    return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
                }
                
                let shouldSetPadding = args["shouldSetPadding"] as! Bool
                FLHVDocsCapture.setShouldAddPadding(shouldSetPadding: shouldSetPadding)
            }
            else if (call.method.caseInsensitiveCompare(FLHVDocsCapture.DOCS_CAPTURE_SET_SHOULD_SHOW_FLASH_ICON) == .orderedSame){
                guard let args = call.arguments as? Dictionary<String, Any> else {
                    return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
                }
                
                let shouldShowFlashIcon = args["shouldShowFlashIcon"] as! Bool
                FLHVDocsCapture.setShouldShowFlashIcon(shouldShowFlashIcon: shouldShowFlashIcon)
            }
            else if (call.method.caseInsensitiveCompare(FLHVDocsCapture.DOCS_CAPTURE_SET_SHOULD_SHOW_REVIEW_SCREEN) == .orderedSame){
                guard let args = call.arguments as? Dictionary<String, Any> else {
                    return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
                }
                
                let shouldShowReviewScreen = args["shouldShowReviewScreen"] as! Bool
                FLHVDocsCapture.setShouldShowReviewScreen(shouldShowReviewScreen: shouldShowReviewScreen)
            }
            else if (call.method.caseInsensitiveCompare(FLHVDocsCapture.DOCS_CAPTURE_SET_DOC_CAPTURE_SUB_TEXT) == .orderedSame){
                guard let args = call.arguments as? Dictionary<String, Any> else {
                    return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
                }
                
                let subText = args["subText"] as! String
                FLHVDocsCapture.setDocCaptureSubText(subText: subText)
            }
            else if (call.method.caseInsensitiveCompare(FLHVDocsCapture.DOCS_CAPTURE_SET_DOC_CAPTURE_DESCRIPTION) == .orderedSame){
                guard let args = call.arguments as? Dictionary<String, Any> else {
                    return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
                }
                
                let description = args["description"] as! String
                FLHVDocsCapture.setDocCaptureDescription(description: description)
            }
            else if (call.method.caseInsensitiveCompare(FLHVDocsCapture.DOCS_CAPTURE_SET_SHOULD_SHOW_INSTRUCTION_PAGE) == .orderedSame){
                guard let args = call.arguments as? Dictionary<String, Any> else {
                    return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
                }
                
                let shouldShowInstructionPage = args["shouldShowInstructionPage"] as! Bool
                FLHVDocsCapture.setShouldShowInstructionPage(shouldShowInstructionPage: shouldShowInstructionPage)
            }
            //        else if (call.method.caseInsensitiveCompare("docsCaptureGetConfig") == .orderedSame){
            //            result(FLHVDocsCapture.getDocConfig())
            //        }
            else if (call.method.caseInsensitiveCompare(FLHVDocsCapture.DOCS_CAPTURE_SET_DOCUMENT_TYPE) == .orderedSame){
                guard let args = call.arguments as? Dictionary<String, Any> else {
                    return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
                }
                
                let docTypeValue = args["docTypeValue"] as! String
                FLHVDocsCapture.setDocumentType(docTypeValue: docTypeValue)
            }
            else if (call.method.caseInsensitiveCompare(FLHVDocsCapture.DOCS_CAPTURE_SET_DOC_CAPTURE_TITLE) == .orderedSame){
                guard let args = call.arguments as? Dictionary<String, Any> else {
                    return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
                }
                
                let titleText = args["titleText"] as! String
                FLHVDocsCapture.setDocCaptureTitle(titleText: titleText)
            }
            else if (call.method.caseInsensitiveCompare(FLHVDocsCapture.DOCS_CAPTURE_SET_DOC_REVIEW_TITLE) == .orderedSame){
                guard let args = call.arguments as? Dictionary<String, Any> else {
                    return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
                }
                
                let docReviewTitle = args["docReviewTitle"] as! String
                FLHVDocsCapture.setDocReviewTitle(docReviewTitle: docReviewTitle)
            }
            else if (call.method.caseInsensitiveCompare(FLHVDocsCapture.DOCS_CAPTURE_SET_DOC_REVIEW_DESCRIPTION) == .orderedSame){
                guard let args = call.arguments as? Dictionary<String, Any> else {
                    return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
                }
                
                let docReviewDescription = args["docReviewDescription"] as! String
                FLHVDocsCapture.setDocReviewDescription(docReviewDescription: docReviewDescription)
            }
            else if (call.method.caseInsensitiveCompare(FLHVDocsCapture.DOCS_CAPTURE_SET_SHOULD_ENABLE_RETRIES) == .orderedSame){
                guard let args = call.arguments as? Dictionary<String, Any> else {
                    return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
                }
                
                let shouldEnableRetries = args["shouldEnableRetries"] as! Bool
                FLHVDocsCapture.setShouldHandleRetries(shouldHandleRetries: shouldEnableRetries)
            }
            else if (call.method.caseInsensitiveCompare(FLHVDocsCapture.DOCS_CAPTURE_SET_UI_STRINGS) == .orderedSame){
                guard let args = call.arguments as? Dictionary<String, Any> else {
                    return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
                }
                
                let uiStrings = args["uiStrings"] as! Dictionary<String, Any>
                FLHVDocsCapture.setUIStrings(uiStrings: uiStrings)
                
            }
            else if (call.method.caseInsensitiveCompare(FLHVDocsCapture.DOCS_CAPTURE_START) == .orderedSame){
                let completionHandler: (_ error: HVError?,_ response: HVResponse?,_ viewController: UIViewController)-> Void = {error, hvResult, vcNew in
                    
                    var errorObj = [String : Any]()
                    var resultObj = [String : String]()
                    
                    if(error != nil){/*Handle error*/
                        //Error Code: error.errorCode, Error Message: error.errorMessage
                        errorObj["errorCode"] = error?.errorCode
                        errorObj["errorMessage"] = error?.errorMessage
                        
                    } else {
                        if let result = hvResult {
                            do{
                                if
                                    let response = result.apiResult {
                                    
                                    let jsonData = try JSONSerialization.data(withJSONObject: response, options: JSONSerialization.WritingOptions.prettyPrinted)
                                    let jsonString = String(data: jsonData, encoding: String.Encoding.utf8)!
                                    
                                    //                                let jsonString = response?.description
                                    
                                    resultObj["apiResult"] = jsonString
                                } else{
                                    resultObj["apiResult"] = "{}"
                                }
                                
                                if let imageUri = result.imageUri {
                                    if !imageUri.isEmpty {
                                        resultObj["imageUri"] = imageUri
                                    }
                                }
                                
                                if let fullImageUri = result.fullImageUri {
                                    if !fullImageUri.isEmpty{
                                        resultObj["fullImageUri"] = fullImageUri
                                    }
                                }
                                
                                if let retakeMessage = result.retakeMessage {
                                    if !retakeMessage.isEmpty {
                                        resultObj["retakeMessage"] = retakeMessage
                                    }
                                }
                                
                                if let action = result.action {
                                    if !action.isEmpty {
                                        resultObj["action"] = action
                                    }
                                }
                                
                                if let headers = result.apiHeaders {
                                    let headersData = try JSONSerialization.data(withJSONObject: headers , options: JSONSerialization.WritingOptions.prettyPrinted)
                                    let headersString = String(data: headersData, encoding: String.Encoding.utf8)
                                    
                                    //                                    let headersString = headers.description
                                    
                                    resultObj["apiHeaders"] = headersString
                                    
                                }else{
                                    resultObj["apiHeaders"] = "{}"
                                }
                            }
                            catch let myJSONError {
                                print(myJSONError)
                            }
                        }
                        
                        
                        
                    }
                    
                    let map = [
                        "errorObj": errorObj,
                        "resultObj": resultObj,
                    ]
                    
                    result(map)
                }
                
                if let topController = UIApplication.shared.keyWindow?.rootViewController{
                    FLHVDocsCapture.start(context: topController, callback: completionHandler)
                }
                
            }
            else{
                result(FlutterMethodNotImplemented)
            }
        })
        
        
        // Handler for FLHVFaceCapture
        hvFaceCaptureChannel.setMethodCallHandler({
            (call: FlutterMethodCall, result: @escaping FlutterResult) -> Void in
            
            if (call.method.caseInsensitiveCompare(FLHVFaceCapture.FACE_CAPTURE_START) == .orderedSame){
                let completionHandler: (_ error: HVError?,_ response: HVResponse?,_ viewController: UIViewController)-> Void = {error, hvResult, vcNew in
                    
                    var errorObj = [String : Any]()
                    var resultObj = [String : String]()
                    
                    if(error != nil){/*Handle error*/
                        //Error Code: error.errorCode, Error Message: error.errorMessage
                        errorObj["errorCode"] = error?.errorCode
                        errorObj["errorMessage"] = error?.errorMessage
                        
                    } else {
                        if let result = hvResult {
                            
                            
                            do{
                                if let response = result.apiResult {
                                    let jsonData = try JSONSerialization.data(withJSONObject: response, options: JSONSerialization.WritingOptions.prettyPrinted)
                                    let jsonString = String(data: jsonData, encoding: String.Encoding.utf8)!
                                    
                                    //                                let jsonString = response?.description
                                    resultObj["apiResult"] = jsonString
                                }else{
                                    resultObj["apiResult"] = "{}"
                                }
                                
                                if let imageUri = result.imageUri {
                                    if !imageUri.isEmpty {
                                        resultObj["imageUri"] = imageUri
                                    }
                                }
                                
                                if let fullImageUri = result.fullImageUri {
                                    if !fullImageUri.isEmpty{
                                        resultObj["fullImageUri"] = fullImageUri
                                    }
                                }
                                
                                if let retakeMessage = result.retakeMessage {
                                    if !retakeMessage.isEmpty {
                                        resultObj["retakeMessage"] = retakeMessage
                                    }
                                }
                                
                                if let action = result.action {
                                    if !action.isEmpty {
                                        resultObj["action"] = action
                                    }
                                }
                                
                                if let headers = result.apiHeaders {
                                    let headersData = try JSONSerialization.data(withJSONObject: headers, options: JSONSerialization.WritingOptions.prettyPrinted)
                                    let headersString = String(data: headersData, encoding: String.Encoding.utf8)
                                    
                                    //                                    let headersString = headers.description
                                    
                                    resultObj["apiHeaders"] = headersString
                                    
                                }else{
                                    resultObj["apiHeaders"] = "{}"
                                }
                            }
                            catch let myJSONError {
                                print(myJSONError)
                            }
                        }
                        
                    }
                    
                    let map = [
                        "errorObj": errorObj,
                        "resultObj": resultObj,
                    ]
                    
                    result(map)
                }
                
                if let topController = UIApplication.shared.keyWindow?.rootViewController{
                    FLHVFaceCapture.start(context: topController, callback: completionHandler)
                }
            }
            else if (call.method.caseInsensitiveCompare(FLHVFaceCapture.FACE_CAPTURE_SET_LIVENESS_ENDPOINT) == .orderedSame){
                guard let args = call.arguments as? Dictionary<String, Any> else {
                    return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
                }
                
                let livenessEndpoint = args["livenessEndpoint"] as! String
                FLHVFaceCapture.setLivenessEndpoint(livenessEndpoint: livenessEndpoint)
            }
            else if (call.method.caseInsensitiveCompare(FLHVFaceCapture.FACE_CAPTURE_SET_SHOULD_RETURN_FULL_IMAGE_URL) == .orderedSame){
                guard let args = call.arguments as? Dictionary<String, Any> else {
                    return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
                }
                
                let shouldReturnFullImageUrl = args["shouldReturnFullImageUrl"] as! Bool
                FLHVFaceCapture.setShouldReturnFullImageUri(shouldReturnFullImageUri: shouldReturnFullImageUrl)
            }
            else if (call.method.caseInsensitiveCompare(FLHVFaceCapture.FACE_CAPTURE_SET_SHOULD_SHOW_CAMERA_SWITCH_BUTTON) == .orderedSame){
                guard let args = call.arguments as? Dictionary<String, Any> else {
                    return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
                }
                
                let shouldShow = args["shouldShow"] as! Bool
                FLHVFaceCapture.setShouldShowCameraSwitchButton(shouldShow: shouldShow)
            }
            else if (call.method.caseInsensitiveCompare(FLHVFaceCapture.FACE_CAPTURE_SET_SHOULD_SHOW_INSTRUCTION_PAGE) == .orderedSame){
                guard let args = call.arguments as? Dictionary<String, Any> else {
                    return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
                }
                
                let shouldShowInstructionPage = args["shouldShowInstructionPage"] as! Bool
                FLHVFaceCapture.setShouldShowInstructionPage(shouldShowInstructionPage: shouldShowInstructionPage)
            }
            else if (call.method.caseInsensitiveCompare(FLHVFaceCapture.FACE_CAPTURE_SET_LIVENESS_MODE) == .orderedSame){
                guard let args = call.arguments as? Dictionary<String, Any> else {
                    return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
                }
                
                let livenessValue = args["livenessValue"] as! String
                FLHVFaceCapture.setLivenessMode(livenessValue: livenessValue)
            }
            else if (call.method.caseInsensitiveCompare(FLHVFaceCapture.FACE_CAPTURE_SET_FACE_CAPTURE_TITLE) == .orderedSame){
                guard let args = call.arguments as? Dictionary<String, Any> else {
                    return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
                }
                
                let faceCaptureTitle = args["faceCaptureTitle"] as! String
                FLHVFaceCapture.setFaceCaptureTitle(faceCaptureTitle: faceCaptureTitle)
            }
            else if (call.method.caseInsensitiveCompare(FLHVFaceCapture.FACE_CAPTURE_SET_SHOULD_USE_BACK_CAMERA) == .orderedSame){
                guard let args = call.arguments as? Dictionary<String, Any> else {
                    return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
                }
                
                let shouldUseBackCamera = args["shouldUseBackCamera"] as! Bool
                FLHVFaceCapture.setShouldUseBackCamera(shouldUseBackCamera: shouldUseBackCamera)
            }
            else if (call.method.caseInsensitiveCompare(FLHVFaceCapture.FACE_CAPTURE_SET_SHOULD_ADD_PADDING) == .orderedSame){
                guard let args = call.arguments as? Dictionary<String, Any> else {
                    return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
                }
                
                let shouldSetPadding = args["shouldSetPadding"] as! Bool
                FLHVFaceCapture.setShouldAddPadding(shouldSetPadding: shouldSetPadding)
            }
            else if (call.method.caseInsensitiveCompare(FLHVFaceCapture.FACE_CAPTURE_SET_PADDING) == .orderedSame){
                guard let args = call.arguments as? Dictionary<String, Any> else {
                    return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
                }
                
                let leftPadding = args["leftPadding"] as! Double
                let rightPadding = args["rightPadding"] as! Double
                let topPadding = args["topPadding"] as! Double
                let bottomPadding = args["bottomPadding"] as! Double
                FLHVFaceCapture.setPadding(leftPadding: leftPadding, rightPadding: rightPadding, topPadding: topPadding, bottomPadding: bottomPadding)
            }
            else if (call.method.caseInsensitiveCompare(FLHVFaceCapture.FACE_CAPTURE_SET_LIVENESS_API_PARAMETERS) == .orderedSame){
                guard let args = call.arguments as? Dictionary<String, Any> else {
                    return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
                }
                
                let params = args["params"] as! String
                FLHVFaceCapture.setLivenessAPIParameters(params: params)
            }
            else if (call.method.caseInsensitiveCompare(FLHVFaceCapture.FACE_CAPTURE_SET_LIVENESS_API_HEADERS) == .orderedSame){
                guard let args = call.arguments as? Dictionary<String, Any> else {
                    return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
                }
                
                let headers = args["headers"] as! String
                FLHVFaceCapture.setLivenessAPIHeaders(headers: headers)
            }
            else if (call.method.caseInsensitiveCompare(FLHVFaceCapture.FACE_CAPTURE_SET_SHOULD_HANDLE_RETRIES) == .orderedSame){
                guard let args = call.arguments as? Dictionary<String, Any> else {
                    return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
                }
                
                let shouldHandleRetries = args["shouldHandleRetries"] as! Bool
                FLHVFaceCapture.setShouldHandleRetries(shouldHandleRetries: shouldHandleRetries)
            }
            else if (call.method.caseInsensitiveCompare(FLHVFaceCapture.FACE_CAPTURE_SET_UI_STRINGS) == .orderedSame){
                guard let args = call.arguments as? Dictionary<String, Any> else {
                    return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
                }
                
                let uiStrings = args["uiStrings"] as! Dictionary<String, Any>
                FLHVFaceCapture.setUIStrings(uiStrings: uiStrings)
                
            }
            else{
                result(FlutterMethodNotImplemented)
            }
        })
        
        
        
        
        // Handler for FLHVNetworkHelper
        hvNetworkHelperChannel.setMethodCallHandler({
            (call: FlutterMethodCall, result: @escaping FlutterResult) -> Void in
            
            if (call.method.caseInsensitiveCompare(FLHVNetworkHelper.NETWORK_HELPER_MAKE_OCR_CALL) == .orderedSame){
                guard let args = call.arguments as? Dictionary<String, Any> else {
                    //result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
                    return
                }
                let endpoint = args["endpoint"] as? String
                let documentUri = args["documentUri"] as? String
                let parameters = args["parameters"] as? NSDictionary
                let headers = args["headers"] as? NSDictionary
                let completionHandler: (_ error: HVError?,_ response: HVResponse?)-> Void = {error, hvResult in
                    
                    var errorObj = [String : Any]()
                    var resultObj = [String : String]()
                    
                    if(error != nil){/*Handle error*/
                        //Error Code: error.errorCode, Error Message: error.errorMessage
                        errorObj["errorCode"] = error?.errorCode
                        errorObj["errorMessage"] = error?.errorMessage
                        
                    } else {
                        if let result = hvResult {
                            
                            do{
                                if                             let response = result.apiResult
                                {
                                    let jsonData = try JSONSerialization.data(withJSONObject: response, options: JSONSerialization.WritingOptions.prettyPrinted)
                                    let jsonString = String(data: jsonData, encoding: String.Encoding.utf8)!
                                    //                            let jsonString = response?.description
                                    
                                    resultObj["apiResult"] = jsonString
                                } else{
                                    resultObj["apiResult"] = "{}"
                                    
                                }
                                
                                if let imageUri = result.imageUri {
                                    if !imageUri.isEmpty {
                                        resultObj["imageUri"] = imageUri
                                    }
                                }
                                
                                if let fullImageUri = result.fullImageUri {
                                    if !fullImageUri.isEmpty{
                                        resultObj["fullImageUri"] = fullImageUri
                                    }
                                }
                                
                                if let retakeMessage = result.retakeMessage {
                                    if !retakeMessage.isEmpty {
                                        resultObj["retakeMessage"] = retakeMessage
                                    }
                                }
                                
                                if let action = result.action {
                                    if !action.isEmpty {
                                        resultObj["action"] = action
                                    }
                                }
                                
                                if let headers = result.apiHeaders {
                                    let headersData = try JSONSerialization.data(withJSONObject: headers, options: JSONSerialization.WritingOptions.prettyPrinted)
                                    let headersString = String(data: headersData, encoding: String.Encoding.utf8)
                                    
                                    //                                let headersString = headers.description
                                    
                                    resultObj["apiHeaders"] = headersString
                                    
                                }else{
                                    resultObj["apiHeaders"] = "{}"
                                    
                                }
                            }
                            catch let myJSONError {
                                print(myJSONError)
                            }
                        }
                        
                    }
                    
                    let map = [
                        "errorObj": errorObj,
                        "resultObj": resultObj,
                    ]
                    
                    result(map)
                }
                
                
                if let topController = UIApplication.shared.keyWindow?.rootViewController{
                    FLHVNetworkHelper.makeOCRCall(context: topController, endpoint: endpoint!, documentUri: documentUri!, parameters: parameters!, headers: headers!, callback: completionHandler)
                }
                
                
                
            }
            else if (call.method.caseInsensitiveCompare(FLHVNetworkHelper.NETWORK_HELPER_MAKE_FACE_MATCH_CALL) == .orderedSame){
                guard let args = call.arguments as? Dictionary<String, Any> else {
                    //result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
                    return
                }
                let endpoint = args["endpoint"] as? String
                let documentUri = args["documentUri"] as? String
                let faceUri = args["faceUri"] as? String
                let parameters = args["parameters"] as? NSDictionary
                let headers = args["headers"] as? NSDictionary
                let completionHandler: (_ error: HVError?,_ response: HVResponse?)-> Void = {error, hvResult in
                    
                    var errorObj = [String : Any]()
                    var resultObj = [String : String]()
                    
                    if(error != nil){/*Handle error*/
                        //Error Code: error.errorCode, Error Message: error.errorMessage
                        errorObj["errorCode"] = error?.errorCode
                        errorObj["errorMessage"] = error?.errorMessage
                        
                    } else {
                        if let result = hvResult {
                            
                            do{
                                if
                                    let response = result.apiResult{
                                    let jsonData = try JSONSerialization.data(withJSONObject: response ?? {}, options: JSONSerialization.WritingOptions.prettyPrinted)
                                    let jsonString = String(data: jsonData, encoding: String.Encoding.utf8)!
                                    //                            let jsonString = response?.description
                                    
                                    resultObj["apiResult"] = jsonString
                                } else{
                                    resultObj["apiResult"] = "{}"
                                    
                                }
                                
                                if let imageUri = result.imageUri {
                                    if !imageUri.isEmpty {
                                        resultObj["imageUri"] = imageUri
                                    }
                                }
                                
                                if let fullImageUri = result.fullImageUri {
                                    if !fullImageUri.isEmpty{
                                        resultObj["fullImageUri"] = fullImageUri
                                    }
                                }
                                
                                if let retakeMessage = result.retakeMessage {
                                    if !retakeMessage.isEmpty {
                                        resultObj["retakeMessage"] = retakeMessage
                                    }
                                }
                                
                                if let action = result.action {
                                    if !action.isEmpty {
                                        resultObj["action"] = action
                                    }
                                }
                                
                                if let headers = result.apiHeaders {
                                    let headersData = try JSONSerialization.data(withJSONObject: headers, options: JSONSerialization.WritingOptions.prettyPrinted)
                                    let headersString = String(data: headersData, encoding: String.Encoding.utf8)
                                    
                                    //                                let headersString = headers.description
                                    
                                    resultObj["apiHeaders"] = headersString
                                    
                                } else{
                                    resultObj["apiHeaders"] = "{}"
                                    
                                }
                            }
                            catch let myJSONError {
                                print(myJSONError)
                            }
                        }
                        
                    }
                    
                    let map = [
                        "errorObj": errorObj,
                        "resultObj": resultObj,
                    ]
                    
                    result(map)
                }
                
                
                if let topController = UIApplication.shared.keyWindow?.rootViewController{
                    FLHVNetworkHelper.makeFaceMatchCall(context: topController, endpoint: endpoint!, faceUri: faceUri!, documentUri: documentUri!, parameters: parameters!, headers: headers!, callback: completionHandler)
                }
                
            }
            else{
                result(FlutterMethodNotImplemented)
            }
        })
        
        hvQRScanCaptureChannel.setMethodCallHandler({
            (call: FlutterMethodCall, result: @escaping FlutterResult) -> Void in
            
            if(call.method.caseInsensitiveCompare(FLHVQRScanCapture.QR_SCAN_CAPTURE_START) == .orderedSame){
                
                /*let completionHandler: (_ error: HVError?, _ result: NSDictionary?) -> Void = { error, result in
                 // What to do, What not to do
                 
                 }
                 
                 if let topController = UIApplication.shared.keyWindow?.rootViewController{
                 //                FLHVQRScanCapture.start(context: topController, callback: completionHandler)
                 }*/
                result(FlutterMethodNotImplemented)
            } else{
                result(FlutterMethodNotImplemented)
            }
            
            
        })
        
        //    let instance = SwiftHvSdkWrapperPlugin()
        //    registrar.addMethodCallDelegate(instance, channel: channel)
    }
    
    
    //    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
    //       if (call.method == "getPlatformVersion") {
    //               result("iOS " + UIDevice.current.systemVersion)
    //           }
    //           else if (call.method == "showAlertDialog") {
    //               DispatchQueue.main.async {
    //                   let alert = UIAlertController(title: "Alert", message: "Hi, My name is flutter", preferredStyle: .alert);
    //                alert.addAction(UIAlertAction(title: "OK", style: UIAlertAction.Style.default, handler: nil))
    //                   UIApplication.shared.keyWindow?.rootViewController?.present(alert, animated: true, completion: nil);
    //               }
    //           }
    //     }
}
