import 'package:flutter/services.dart';

import 'HVMethodCallStrings.dart';

class HVDocsCapture {
  static const MethodChannel _hvDocsCaptureChannel =
      const MethodChannel(HVMethodCallStrings.DOCS_CAPTURE_CHANNEL);

  /// docsCaptureSetOCRAPIDetails() is used to pre-set the endpoint, side of the document, params, headers to make OCR call
  ///     immediately after the document has been captured
  ///
  /// params :
  ///     ocrEndpoint : type - String, value - endpoint URL which will be used to make ocr call
  ///                                               by default, an endpoint to make ocr calls has been assigned
  ///                                                           inside the sdk itself
  ///     documentSideString : type - String, value - one of [FRONT, BACK] to specify the side of the document captured
  ///     params : type - Map, value - map/json containing key-value pairs of different fraud/malice checks provided
  ///                                     by Hyperverge.co
  ///     headers : type - Map, value - map/json containing key-value pairs of different headers options provided
  ///                                     by Hyperverge.co
  /// returns :
  ///     void
  static docsCaptureSetOCRAPIDetails(String ocrEndpoint,
      String documentSideString, Map params, Map headers) async {
    try {
      await _hvDocsCaptureChannel
          .invokeMethod(HVMethodCallStrings.DOCS_CAPTURE_SET_OCR_API_DETAILS, {
        "ocrEndpoint": ocrEndpoint,
        "documentSideString": documentSideString,
        "params": params,
        "headers": headers
      });
    } catch (e) {
      print(e.toString());
    }
  }

  /// docsCaptureSetAspectRatio() is used to set the aspect ratio of the document to be captured
  ///
  /// params :
  ///     aspectRatioValue : type - double
  /// returns :
  ///     void
  static docsCaptureSetAspectRatio(double aspectRatioValue) async {
    try {
      _hvDocsCaptureChannel.invokeMethod(
          HVMethodCallStrings.DOCS_CAPTURE_SET_ASPECT_RATIO,
          {"aspectRatioValue": aspectRatioValue});
    } catch (e) {
      print(e.toString());
    }
  }

  /// docsCaptureSetShouldAddPadding() is set to true if the image has to padded
  ///
  /// params :
  ///     aspectRatioValue : type - bool
  /// returns :
  ///     void
  static docsCaptureSetShouldAddPadding(bool shouldSetPadding) async {
    try {
      _hvDocsCaptureChannel.invokeMethod(
          HVMethodCallStrings.DOCS_CAPTURE_SET_SHOULD_ADD_PADDING,
          {"shouldSetPadding": shouldSetPadding});
    } catch (e) {
      print(e.toString());
    }
  }

  /// docsCaptureSetShouldShowFlashIcon() is set to true if flash icon has to be shown to the user
  ///     in the client's application
  ///
  /// params :
  ///     shouldShowFlashIcon : type - bool
  /// returns :
  ///     void
  static docsCaptureSetShouldShowFlashIcon(bool shouldShowFlashIcon) async {
    try {
      _hvDocsCaptureChannel.invokeMethod(
          HVMethodCallStrings.DOCS_CAPTURE_SET_SHOULD_SHOW_FLASH_ICON,
          {"shouldShowFlashIcon": shouldShowFlashIcon});
    } catch (e) {
      print(e.toString());
    }
  }

  /// docsCaptureSetShouldShowReviewScreen() is set to true if review screen has to be shown to the user
  ///     in the client's application
  ///
  /// params :
  ///     shouldShowReviewScreen : type - bool
  /// returns :
  ///     void
  static docsCaptureSetShouldShowReviewScreen(
      bool shouldShowReviewScreen) async {
    try {
      _hvDocsCaptureChannel.invokeMethod(
          HVMethodCallStrings.DOCS_CAPTURE_SET_SHOULD_SHOW_REVIEW_SCREEN,
          {"shouldShowReviewScreen": shouldShowReviewScreen});
    } catch (e) {
      print(e.toString());
    }
  }

  /// docsCaptureSetDocCaptureSubText() is used to set the sub text inside document capture screen
  ///
  /// params :
  ///     subText : type - String
  /// returns :
  ///     void
  static docsCaptureSetDocCaptureSubText(String subText) async {
    try {
      _hvDocsCaptureChannel.invokeMethod(
          HVMethodCallStrings.DOCS_CAPTURE_SET_DOC_CAPTURE_SUB_TEXT,
          {"subText": subText});
    } catch (e) {
      print(e.toString());
    }
  }

  /// docsCaptureSetDocCaptureDescription() is used to set the description inside document capture screen
  ///
  /// params :
  ///     description : type - String
  /// returns :
  ///     void
  static docsCaptureSetDocCaptureDescription(String description) async {
    try {
      _hvDocsCaptureChannel.invokeMethod(
          HVMethodCallStrings.DOCS_CAPTURE_SET_DOC_CAPTURE_DESCRIPTION,
          {"description": description});
    } catch (e) {
      print(e.toString());
    }
  }

  /// docsCaptureSetShouldShowInstructionPage() is set to true if instructions page has to be shown to the user
  ///     in the client's application
  ///
  /// params :
  ///     shouldShowInstructionPage : type - bool
  /// returns :
  ///     void
  static docsCaptureSetShouldShowInstructionPage(
      bool shouldShowInstructionPage) async {
    try {
      _hvDocsCaptureChannel.invokeMethod(
          HVMethodCallStrings.DOCS_CAPTURE_SET_SHOULD_SHOW_INSTRUCTION_PAGE,
          {"shouldShowInstructionPage": shouldShowInstructionPage});
    } catch (e) {
      print(e.toString());
    }
  }

  /// docsCaptureSetDocumentType()  is used to set the type of the document which has to be captured
  ///
  /// params :
  ///     docTypeValue : type - String, value - one of [CARD, A4, PASSPORT, OTHER]
  ///                                           by default, CARD type is initialized in sdk
  /// returns :
  ///     void
  static docsCaptureSetDocumentType(String docTypeValue) async {
    try {
      _hvDocsCaptureChannel.invokeMethod(
          HVMethodCallStrings.DOCS_CAPTURE_SET_DOCUMENT_TYPE,
          {"docTypeValue": docTypeValue});
    } catch (e) {
      print(e.toString());
    }
  }

  /// docsCaptureSetDocCaptureTitle() is used to set the title inside document capture screen
  ///
  /// params :
  ///     titleText : type - String
  /// returns :
  ///     void
  static docsCaptureSetDocCaptureTitle(String titleText) async {
    try {
      _hvDocsCaptureChannel.invokeMethod(
          HVMethodCallStrings.DOCS_CAPTURE_SET_DOC_CAPTURE_TITLE,
          {"titleText": titleText});
    } catch (e) {
      print(e.toString());
    }
  }

  /// docsCaptureSetDocReviewTitle() is used to set the title inside document review screen
  ///
  /// params :
  ///     docReviewTitle : type - String
  /// returns :
  ///     void
  static docsCaptureSetDocReviewTitle(String docReviewTitle) async {
    try {
      _hvDocsCaptureChannel.invokeMethod(
          HVMethodCallStrings.DOCS_CAPTURE_SET_DOC_REVIEW_TITLE,
          {"docReviewTitle": docReviewTitle});
    } catch (e) {
      print(e.toString());
    }
  }

  /// docsCaptureSetDocReviewDescription() is used to set the description inside document review screen
  ///
  /// params :
  ///     docReviewDescription : type - String
  /// returns :
  ///     void
  static docsCaptureSetDocReviewDescription(String docReviewDescription) async {
    try {
      _hvDocsCaptureChannel.invokeMethod(
          HVMethodCallStrings.DOCS_CAPTURE_SET_DOC_REVIEW_DESCRIPTION,
          {"docReviewDescription": docReviewDescription});
    } catch (e) {
      print(e.toString());
    }
  }

  /// docsCaptureSetShouldEnableRetries() is used to set whether HyperSnapSDK should handle retries after any doc capture screen
  ///
  /// params :
  ///     shouldHandleRetries : type - bool
  /// returns :
  ///     void
  static docsCaptureSetShouldEnableRetries(bool shouldEnableRetries) async {
    try {
      _hvDocsCaptureChannel.invokeMethod(
          HVMethodCallStrings.DOCS_CAPTURE_SET_SHOULD_ENABLE_RETRIES,
          {"shouldEnableRetries": shouldEnableRetries});
    } catch (e) {
      print(e.toString());
    }
  }

  /// docsCaptureSetUIStrings() is used to set custom strings to all customisable text in HyperSnapSDK
  ///
  /// params :
  ///     uiStrings : type - Map, value - map of customisable strings keys and values
  /// returns :
  ///     void
  static docsCaptureSetUIStrings(Map uiStrings) async {
    try {
      _hvDocsCaptureChannel.invokeMethod(
          HVMethodCallStrings.DOCS_CAPTURE_SET_UI_STRINGS,
          {"uiStrings": uiStrings});
    } catch (e) {
      print(e.toString());
    }
  }

  /// docsCaptureStart() is used to start docs activity/view in the client's application
  ///
  /// params :
  /// returns :
  ///     Future<Map>, contains the result(HVError and HVResponse) from native hypersnapSDK document capture functions
  static Future<Map> docsCaptureStart() async {
    Map map = Map();
    try {
      map = await _hvDocsCaptureChannel
          .invokeMethod(HVMethodCallStrings.DOCS_CAPTURE_START);
    } catch (e) {
      print(e.toString());
    }
    return map;
  }
}
