import 'package:flutter/services.dart';

import 'HVMethodCallStrings.dart';

class HVFaceCapture {
  static const MethodChannel _hvFaceCaptureChannel =
      const MethodChannel(HVMethodCallStrings.FACE_CAPTURE_CHANNEL);

  /// faceCaptureStart() is used to start face activity/view in the client's application
  ///
  /// params :
  /// returns :
  ///     Future<Map>, contains the result(HVError and HVResponse) from native hypersnapSDK face capture functions
  static Future<Map> faceCaptureStart() async {
    Map map = Map();
    try {
      map = await _hvFaceCaptureChannel
          .invokeMethod(HVMethodCallStrings.FACE_CAPTURE_START);
    } catch (e) {
      print(e.toString());
    }
    return map;
  }

  /// faceCaptureSetLivenessEndpoint() is used to overwrite the default liveness endpoint
  ///
  /// params :
  ///     livenessEndpoint : type - String, value - endpoint URL which will be used to make liveness call
  ///                                               by default, an endpoint to make liveness calls has been assigned
  ///                                                           inside the sdk itself
  /// returns :
  ///     void
  static faceCaptureSetLivenessEndpoint(String livenessEndpoint) async {
    try {
      await _hvFaceCaptureChannel.invokeMethod(
          HVMethodCallStrings.FACE_CAPTURE_SET_LIVENESS_ENDPOINT,
          {"livenessEndpoint": livenessEndpoint});
    } catch (e) {
      print(e.toString());
    }
  }

  /// faceCaptureSetShouldReturnFullImageUrl() is set to true if full image Uri has to be sent inside the result from
  ///     face activity along with other parameters
  ///
  /// params :
  ///     shouldReturnFullImageUrl : type - bool
  /// returns :
  ///     void
  static faceCaptureSetShouldReturnFullImageUrl(
      bool shouldReturnFullImageUrl) async {
    try {
      await _hvFaceCaptureChannel.invokeMethod(
          HVMethodCallStrings.FACE_CAPTURE_SET_SHOULD_RETURN_FULL_IMAGE_URL,
          {"shouldReturnFullImageUrl": shouldReturnFullImageUrl});
    } catch (e) {
      print(e.toString());
    }
  }

  /// faceCaptureSetShouldShowCameraSwitchButton() is set to true if camera switch button has to be shown to the user
  ///     in the client's application
  ///
  /// params :
  ///     shouldShow : type - bool
  /// returns :
  ///     void
  static faceCaptureSetShouldShowCameraSwitchButton(bool shouldShow) async {
    try {
      await _hvFaceCaptureChannel.invokeMethod(
          HVMethodCallStrings.FACE_CAPTURE_SET_SHOULD_SHOW_CAMERA_SWITCH_BUTTON,
          {"shouldShow": shouldShow});
    } catch (e) {
      print(e.toString());
    }
  }

  /// faceCaptureSetShouldShowInstructionPage() is set to true if instructions page has to be shown to the user
  ///     in the client's application
  ///
  /// params :
  ///     shouldShowInstructionPage : type - bool
  /// returns :
  ///     void
  static faceCaptureSetShouldShowInstructionPage(
      bool shouldShowInstructionPage) async {
    try {
      await _hvFaceCaptureChannel.invokeMethod(
          HVMethodCallStrings.FACE_CAPTURE_SET_SHOULD_SHOW_INSTRUCTION_PAGE,
          {"shouldShowInstructionPage": shouldShowInstructionPage});
    } catch (e) {
      print(e.toString());
    }
  }

  /// faceCaptureSetLivenessMode() is used to overwrite the default Liveness Mode
  ///
  /// params :
  ///     livenessValue : type - String, value - one of [NONE, TEXTURELIVENESS], by default, TEXTURELIVENESS is
  ///                                             set inside the sdk itself
  /// returns :
  ///     void
  static faceCaptureSetLivenessMode(String livenessValue) async {
    try {
      await _hvFaceCaptureChannel.invokeMethod(
          HVMethodCallStrings.FACE_CAPTURE_SET_LIVENESS_MODE,
          {"livenessValue": livenessValue});
    } catch (e) {
      print(e.toString());
    }
  }

  /// faceCaptureSetFaceCaptureTitle() is used to set the title inside face capture screen
  ///
  /// params :
  ///     faceCaptureTitle : type - String
  /// returns :
  ///     void
  static faceCaptureSetFaceCaptureTitle(String faceCaptureTitle) async {
    try {
      await _hvFaceCaptureChannel.invokeMethod(
          HVMethodCallStrings.FACE_CAPTURE_SET_FACE_CAPTURE_TITLE,
          {"faceCaptureTitle": faceCaptureTitle});
    } catch (e) {
      print(e.toString());
    }
  }

  /// faceCaptureSetShouldUseBackCamera() is set to true if the face capture screen should start with back camera
  ///     rather than default front camera
  ///
  /// params :
  ///     shouldUseBackCamera : type - bool
  /// returns :
  ///     void
  static faceCaptureSetShouldUseBackCamera(bool shouldUseBackCamera) async {
    try {
      await _hvFaceCaptureChannel.invokeMethod(
          HVMethodCallStrings.FACE_CAPTURE_SET_SHOULD_USE_BACK_CAMERA,
          {"shouldUseBackCamera": shouldUseBackCamera});
    } catch (e) {
      print(e.toString());
    }
  }

  /// faceCaptureSetShouldAddPadding() is set to true if the image has to padded
  ///
  /// params :
  ///     shouldSetPadding : type - bool
  /// returns :
  ///     void
  static faceCaptureSetShouldAddPadding(bool shouldSetPadding) async {
    try {
      await _hvFaceCaptureChannel.invokeMethod(
          HVMethodCallStrings.FACE_CAPTURE_SET_SHOULD_ADD_PADDING,
          {"shouldSetPadding": shouldSetPadding});
    } catch (e) {
      print(e.toString());
    }
  }

  /// faceCaptureSetPadding() is used to set specific amount of padding in all four directions to the image
  ///
  /// params :
  ///     leftPadding : type - double
  ///     rightPadding : type - double
  ///     topPadding : type - double
  ///     bottomPadding : type - double
  /// returns :
  ///     void
  static faceCaptureSetPadding(double leftPadding, double rightPadding,
      double topPadding, double bottomPadding) async {
    try {
      await _hvFaceCaptureChannel
          .invokeMethod(HVMethodCallStrings.FACE_CAPTURE_SET_PADDING, {
        "leftPadding": leftPadding,
        "rightPadding": rightPadding,
        "topPadding": topPadding,
        "bottomPadding": bottomPadding
      });
    } catch (e) {
      print(e.toString());
    }
  }

  /// faceCaptureSetLivenessAPIParameters() is used to set liveness api parameters which will be used to make liveness call
  ///
  /// params :
  ///     params : type - String, value - string containing key-value pairs of different fraud/malice checks provided
  ///                                     by Hyperverge.co
  /// returns :
  ///     void
  static faceCaptureSetLivenessAPIParameters(String params) async {
    try {
      await _hvFaceCaptureChannel.invokeMethod(
          HVMethodCallStrings.FACE_CAPTURE_SET_LIVENESS_API_PARAMETERS,
          {"params": params});
    } catch (e) {
      print(e.toString());
    }
  }

  /// faceCaptureSetLivenessAPIHeaders() is used to set liveness api headers which will be used to make liveness call
  ///
  /// params :
  ///     headers : type - String, value - string containing key-value pairs of different headers options provided
  ///                                     by Hyperverge.co
  /// returns :
  ///     void
  static faceCaptureSetLivenessAPIHeaders(String headers) async {
    try {
      await _hvFaceCaptureChannel.invokeMethod(
          HVMethodCallStrings.FACE_CAPTURE_SET_LIVENESS_API_HEADERS,
          {"headers": headers});
    } catch (e) {
      print(e.toString());
    }
  }

  /// faceCaptureSetShouldHandleRetries() is used to set whether HyperSnapSDK should handle retries after any face capture screen
  ///
  /// params :
  ///     shouldHandleRetries : type - bool
  /// returns :
  ///     void
  static faceCaptureSetShouldHandleRetries(bool shouldHandleRetries) async {
    try {
      await _hvFaceCaptureChannel.invokeMethod(
          HVMethodCallStrings.FACE_CAPTURE_SET_SHOULD_HANDLE_RETRIES,
          {"shouldHandleRetries": shouldHandleRetries});
    } catch (e) {
      print(e.toString());
    }
  }

  /// faceCaptureSetUIStrings() is used to set custom strings to all customisable text in HyperSnapSDK
  ///
  /// params :
  ///     uiStrings : type - Map, value - map of customisable strings keys and values
  /// returns :
  ///     void
  static faceCaptureSetUIStrings(Map uiStrings) async {
    try {
      _hvFaceCaptureChannel.invokeMethod(
          HVMethodCallStrings.FACE_CAPTURE_SET_UI_STRINGS,
          {"uiStrings": uiStrings});
    } catch (e) {
      print(e.toString());
    }
  }
}
