import 'package:flutter/services.dart';

import 'HVMethodCallStrings.dart';

class HVNetworkHelper {
  static const MethodChannel _hvNetworkHelperChannel =
      const MethodChannel(HVMethodCallStrings.NETWORK_HELPER_CHANNEL);

  /// networkHelperMakeOCRCall() is used to make an OCR api call using the document image
  ///
  /// params :
  ///     endpoint : type - String, value - endpoint URL which will be called to obtain ocr response
  ///     documentUri : type - String, value - path/uri of document image on which ocr call has to be performed
  ///     parameters : type - Map, value - map/json containing key-value pairs of different fraud/malice checks provided
  ///                                     by Hyperverge.co
  ///     headers : type - Map, value - map/json containing key-value pairs of different headers options provided
  ///                                     by Hyperverge.co
  /// returns :
  ///     Future<Map>, contains the response obtained after ocr call
  static Future<Map> networkHelperMakeOCRCall(
      String endpoint, String documentUri, Map parameters, Map headers) async {
    final Map map = await _hvNetworkHelperChannel
        .invokeMethod(HVMethodCallStrings.NETWORK_HELPER_MAKE_OCR_CALL, {
      "endpoint": endpoint,
      "documentUri": documentUri,
      "parameters": parameters,
      "headers": headers
    });
    return map;
  }

  /// networkHelperMakeFaceMatchCall() is used to make face match api call using face and document image
  ///
  /// params :
  ///     endpoint : type - String, value - endpoint URL which will be called to obtain ocr response
  ///     faceUri : type - String, value - path/uri of face image on which face match call has to be performed
  ///     documentUri : type - String, value - path/uri of document image on which face match call has to be performed
  ///     parameters : type - Map, value - map/json containing key-value pairs of different fraud/malice checks provided
  ///                                     by Hyperverge.co
  ///     headers : type - Map, value - map/json containing key-value pairs of different headers options provided
  ///                                     by Hyperverge.co
  /// returns :
  ///     Future<Map>, contains the response obtained after face match call
  static Future<Map> networkHelperMakeFaceMatchCall(String endpoint,
      String faceUri, String documentUri, Map parameters, Map headers) async {
    final Map map = await _hvNetworkHelperChannel
        .invokeMethod(HVMethodCallStrings.NETWORK_HELPER_MAKE_FACE_MATCH_CALL, {
      "endpoint": endpoint,
      "faceUri": faceUri,
      "documentUri": documentUri,
      "parameters": parameters,
      "headers": headers
    });
    return map;
  }
}
